#!/usr/bin/env ruby

usage='''
Program: Atlas-SNP2-genotyper: A genotyper for variant sites called by Atlas-SNP2
Version: 1.0
Usage:   Atlas-SNP2-genotyper.rb [postrior probability cutoff] [Atlas-SNP2 result file] > [outputfile]

'''

if ARGV[0] == nil
  puts usage
  exit
end

cutoff = ARGV[0].to_f
atlas_results = open(ARGV[1],'r')


def factorial(n)
  if n <= 1
    return 1
  else
    a = 1
    n.to_i.times do
      a *= n
      n -= 1
    end
    return a
  end
end

def acc_prob(n,k)
  if n < 50
    a = 0
    (k+1).to_i.times do
      a += factorial(n)*(0.5**n)/(factorial(k)*factorial(n-k))
      k -= 1
    end
    return 1-a
  else
    z = ((2*k+0.5) - n)/Math.sqrt(n)
    if z>= 5.99
      return 0.000000001
    elsif z>= 5.61
      return 0.00000001
    elsif z>= 5.19
      return 0.0000001
    elsif z >= 3.719
      return 0.00001
    elsif z >= 3.09
      return 0.0001
    elsif z >=2.32
      return 0.001
    elsif z >= 2.05
      return 0.015
    elsif z >=1.88
      return 0.025
    elsif z >=1.75
      return 0.035
    elsif z >= 1.64
      return 0.05
    elsif z >= 1.28
      return 0.08
    elsif z >=0.84
      return 0.15
    elsif z >=0.52
      return 0.15
    elsif z >= 0.25
      return 0.15
    elsif z >= 0
      return 0.45
    end

  end

end


def passes_strandedness_test(reads_info, num_alt_reads,total_cov )
  return true if total_cov.to_i < 8 #with so few total reads, it is quite possible they are all one direction
  plus_dir = reads_info.count("+")
  minus_dir = num_alt_reads.to_i - plus_dir
  return false if plus_dir.to_f/num_alt_reads.to_f < 0.01
  return false if minus_dir.to_f/num_alt_reads.to_f < 0.01
  return true
end


atlas_results.each do |line|
  cols = line.split("\t")

  if cols[0] == 'refName'
    print "#{cols[0]}\t#{cols[1]}\t#{cols[2]}\t#{cols[3]}\t#{cols[4]}\t#{cols[5]}\t#{cols[6]}\t#{cols[7]}\t#{cols[15]}\tGenotype\tp\n"
    next
  end

  var_num = cols[5].to_f
  ref_num = cols[7].to_f
  n = ref_num + var_num
  reads_info = cols[18]
  pr = cols[15].to_f

  next if pr < cutoff
  next unless passes_strandedness_test(reads_info, var_num, n)
  
  if var_num == 0 or ref_num == 0 
    p = 0.000000001
  elsif var_num >= ref_num
    p = acc_prob(n, var_num)
  else
    p =   acc_prob(n, ref_num)
  end
  
    
  if var_num/n <= 0.1
    print "#{cols[0]}\t#{cols[1]}\t#{cols[2]}\t#{cols[3]}\t#{cols[4]}\t#{cols[5]}\t#{cols[6]}\t#{cols[7]}\t#{cols[15]}\t0/0\t#{p}\n"
  elsif var_num/n > 0.1 and var_num/n < 0.9
    print "#{cols[0]}\t#{cols[1]}\t#{cols[2]}\t#{cols[3]}\t#{cols[4]}\t#{cols[5]}\t#{cols[6]}\t#{cols[7]}\t#{cols[15]}\t0/1\t#{p}\n"
  else
    print "#{cols[0]}\t#{cols[1]}\t#{cols[2]}\t#{cols[3]}\t#{cols[4]}\t#{cols[5]}\t#{cols[6]}\t#{cols[7]}\t#{cols[15]}\t1/1\t#{p}\n"
  end

end
